package br.com.cuidai.sdk.cuidai_plugin

import android.app.Activity
import android.app.Application
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import androidx.annotation.NonNull
import br.com.cuidai.app_components.extensions.bundleToMap
import br.com.cuidai.app_components.extensions.splitQuery
import br.com.cuidai.base_api.extensions.toBundle
import br.com.cuidai.events.EventsListener
import br.com.cuidai.router.Router.route
import br.com.cuidai.sdk.CuidaiSDK
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** CuidaiPlugin */
class CuidaiPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channelFromFlutter: MethodChannel
    private lateinit var channelToFlutter: MethodChannel
    private lateinit var routeChannelToFlutter: MethodChannel

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channelFromFlutter = MethodChannel(flutterPluginBinding.binaryMessenger, "cuidai_plugin")
        channelToFlutter =
            MethodChannel(flutterPluginBinding.binaryMessenger, "cuidai_plugin_native")
        channelToFlutterAnalytics =
            MethodChannel(flutterPluginBinding.binaryMessenger, "cuidai_plugin_analytics")
        routeChannelToFlutter =
            MethodChannel(flutterPluginBinding.binaryMessenger, "cuidai_plugin_route")
        channelFromFlutter.setMethodCallHandler(this)

        CuidaiSDK.externalRouteCallback = { uri, parameters, _ ->
            val params = parameters.bundleToMap()
            var route = uri
            if (uri.contains("external")) {
                try {
                    splitQuery(uri)["route"]?.let { externalRoute ->
                        route = externalRoute
                    }
                } catch (e: Exception) {
                    print(e)
                }
            }
            if (activity?.isFinishing == true) {
                activity?.baseContext?.startActivity(
                    Intent(Intent.ACTION_VIEW).apply {
                        addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                        data = Uri.parse("vitat://home")
                    }
                )
            }
            activity?.runOnUiThread {
                if (route.startsWith("event.")
                    || route.startsWith("updateHome")
                    || route.startsWith("updateUserData")
                    || route.startsWith("updateUser")
                ) {
                    if (::channelToFlutter.isInitialized) {
                        channelToFlutter.invokeMethod(
                            "route",
                            mapOf("route" to route, "params" to params)
                        )
                    }
                } else {
                    if (::routeChannelToFlutter.isInitialized) {
                        routeChannelToFlutter.invokeMethod(
                            "route",
                            mapOf("route" to route, "params" to params)
                        )
                    }
                }
            }
        }

    }


    companion object {
        private lateinit var channelToFlutterAnalytics: MethodChannel
        private var activity: Activity? = null


        fun initSDK(appName: String, application: Application) {
            CuidaiSDK.init(
                application,
                appName,
                null,
                true
            )


            CuidaiSDK.initEvent(
                application,
                object : EventsListener {
                    override fun sendToAnalytics(
                        event: String,
                        parameters: Map<String, String>,
                        adjustEvent: String?
                    ) {


                        activity?.runOnUiThread {

                            if (::channelToFlutterAnalytics.isInitialized) {

                                channelToFlutterAnalytics.invokeMethod(
                                    "event",
                                    mapOf(
                                        "event" to event,
                                        "params" to parameters
                                    )
                                )

                            }

                        }
                    }
                },
                null
            )
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "initSDK" -> {
                val appName: String =
                    (call.arguments as? HashMap<*, *>)?.get("appName")?.toString() ?: "vitat"
            }
            "getPlatformVersion" -> {
                result.success("Android ${android.os.Build.VERSION.RELEASE}")
            }
            "openChat" -> {
                activity?.route("default")
            }
            "setUserProperty" -> {
                val key: String? = (call.arguments as? HashMap<*, *>)?.get("key")?.toString()
                val value: String? = (call.arguments as? HashMap<*, *>)?.get("value")?.toString()
                key?.let { k ->
                    CuidaiSDK.setUserProperty(k, value)
                }
            }
            "getHeaders" -> {
                var header = CuidaiSDK.getHeaders()
                result.success(header)
            }
            "setUserProperties" -> {
                val properties =
                    (call.arguments as? HashMap<*, *>)?.get("properties") as? Map<String, String?>
                properties?.let {
                    CuidaiSDK.setUserProperties(it)
                }
            }
            "logout" -> {
                CuidaiSDK.logout()
            }
            "route" -> {
                val route: String? = (call.arguments as? HashMap<*, *>)?.get("route")?.toString()
                val params: HashMap<String, String>? =
                    (call.arguments as? HashMap<*, *>)?.get("params") as? HashMap<String, String>
                route?.replace("flutter://", "")?.let {
                    activity?.route(it, params = params?.toBundle() ?: Bundle())
                }
            }


            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channelFromFlutter.setMethodCallHandler(null)
        channelToFlutter.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {}

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {}

    override fun onDetachedFromActivity() {}
}
